MODULE transfert_mod
USE genmod
  
  TYPE t_request
    INTEGER :: type_field
    INTEGER :: max_size
    INTEGER :: size
    INTEGER,POINTER :: src_domain(:)
    INTEGER,POINTER :: src_i(:)
    INTEGER,POINTER :: src_j(:)
    INTEGER,POINTER :: src_ind(:)
    INTEGER,POINTER :: target_ind(:)
    INTEGER,POINTER :: target_i(:)
    INTEGER,POINTER :: target_j(:)
  END TYPE t_request
  
  TYPE(t_request),POINTER :: req_i1(:)
  TYPE(t_request),POINTER :: req_e1(:)
  
CONTAINS

  SUBROUTINE init_transfert
  USE domain_mod
  USE dimensions
  USE field_mod
  USE metric
  IMPLICIT NONE
  INTEGER :: ind,i,j
 
    CALL create_request(field_t,req_i1)

    DO ind=1,ndomain
      CALL swap_dimensions(ind)
      DO i=ii_begin,ii_end+1
        CALL request_add_point(ind,i,jj_begin-1,req_i1)
      ENDDO

      DO j=jj_begin,jj_end
        CALL request_add_point(ind,ii_end+1,j,req_i1)
      ENDDO    
      DO i=ii_begin,ii_end
        CALL request_add_point(ind,i,jj_end+1,req_i1)
      ENDDO    

      DO j=jj_begin,jj_end+1
        CALL request_add_point(ind,ii_begin-1,j,req_i1)
      ENDDO    
    
      DO i=ii_begin,ii_end
        CALL request_add_point(ind,i,jj_begin,req_i1)
      ENDDO

      DO j=jj_begin,jj_end
        CALL request_add_point(ind,ii_end,j,req_i1)
      ENDDO    
    
      DO i=ii_begin,ii_end
        CALL request_add_point(ind,i,jj_end,req_i1)
      ENDDO    

      DO j=jj_begin,jj_end
        CALL request_add_point(ind,ii_begin,j,req_i1)
      ENDDO    
    
    ENDDO
  
  
    CALL create_request(field_u,req_e1)
    DO ind=1,ndomain
      CALL swap_dimensions(ind)
      DO i=ii_begin,ii_end
        CALL request_add_point(ind,i,jj_begin-1,req_e1,rup)
        CALL request_add_point(ind,i+1,jj_begin-1,req_e1,lup)
      ENDDO

      DO j=jj_begin,jj_end
        CALL request_add_point(ind,ii_end+1,j,req_e1,left)
        CALL request_add_point(ind,ii_end+1,j-1,req_e1,lup)
      ENDDO    
    
      DO i=ii_begin,ii_end
        CALL request_add_point(ind,i,jj_end+1,req_e1,ldown)
        CALL request_add_point(ind,i-1,jj_end+1,req_e1,rdown)
      ENDDO    

      DO j=jj_begin,jj_end
        CALL request_add_point(ind,ii_begin-1,j,req_e1,right)
        CALL request_add_point(ind,ii_begin-1,j+1,req_e1,rdown)
      ENDDO   

      DO i=ii_begin+1,ii_end-1
        CALL request_add_point(ind,i,jj_begin,req_e1,right)
        CALL request_add_point(ind,i,jj_end,req_e1,right)
      ENDDO
    
      DO j=jj_begin+1,jj_end-1
        CALL request_add_point(ind,ii_begin,j,req_e1,rup)
        CALL request_add_point(ind,ii_end,j,req_e1,rup)
      ENDDO   

      CALL request_add_point(ind,ii_begin+1,jj_begin,req_e1,left)
      CALL request_add_point(ind,ii_begin,jj_begin+1,req_e1,ldown)
      CALL request_add_point(ind,ii_begin+1,jj_end,req_e1,left)
      CALL request_add_point(ind,ii_end,jj_begin+1,req_e1,ldown)
     
    ENDDO
  
  END SUBROUTINE init_transfert
  
  SUBROUTINE create_request(type_field,request)
  USE domain_mod
  USE field_mod
  IMPLICIT NONE
    INTEGER :: type_field
    TYPE(t_request),POINTER :: request(:)
    TYPE(t_request),POINTER :: req
    TYPE(t_domain),POINTER :: d
    INTEGER :: ind
    INTEGER :: max_size
        
    ALLOCATE(request(ndomain))

    DO ind=1,ndomain
      req=>request(ind)
      d=>domain(ind)
      IF (type_field==field_t) THEN
        Max_size=2*(d%iim+2)+2*(d%jjm+2)
      ELSE IF (type_field==field_u) THEN
        Max_size=3*(2*(d%iim+2)+2*(d%jjm+2))
      ELSE IF (type_field==field_z) THEN
        Max_size=2*(2*(d%iim+2)+2*(d%jjm+2))
      ENDIF

      req%type_field=type_field
      req%max_size=max_size*2
      req%size=0
      ALLOCATE(req%src_domain(req%max_size))
      ALLOCATE(req%src_ind(req%max_size))
      ALLOCATE(req%target_ind(req%max_size))
      ALLOCATE(req%src_i(req%max_size))
      ALLOCATE(req%src_j(req%max_size))
      ALLOCATE(req%target_i(req%max_size))
      ALLOCATE(req%target_j(req%max_size))
    ENDDO
  
  END SUBROUTINE create_request

  SUBROUTINE reallocate_request(req)
  IMPLICIT NONE
    TYPE(t_request),POINTER :: req
      
    INTEGER,POINTER :: src_domain(:)
    INTEGER,POINTER :: src_ind(:)
    INTEGER,POINTER :: target_ind(:)
    INTEGER,POINTER :: src_i(:)
    INTEGER,POINTER :: src_j(:)
    INTEGER,POINTER :: target_i(:)
    INTEGER,POINTER :: target_j(:)

    PRINT *,"REALLOCATE_REQUEST"
    src_domain=>req%src_domain
    src_ind=>req%src_ind
    target_ind=>req%target_ind
    src_i=>req%src_i
    src_j=>req%src_j
    target_i=>req%target_i
    target_j=>req%target_j
!    req%max_size=req%max_size*2
    ALLOCATE(req%src_domain(req%max_size*2))
    ALLOCATE(req%src_ind(req%max_size*2))
    ALLOCATE(req%target_ind(req%max_size*2))
    ALLOCATE(req%src_i(req%max_size*2))
    ALLOCATE(req%src_j(req%max_size*2))
    ALLOCATE(req%target_i(req%max_size*2))
    ALLOCATE(req%target_j(req%max_size*2))
    
    req%src_domain(1:req%max_size)=src_domain(:)
    req%src_ind(1:req%max_size)=src_ind(:)
    req%target_ind(1:req%max_size)=target_ind(:)
    req%src_i(1:req%max_size)=src_i(:)
    req%src_j(1:req%max_size)=src_j(:)
    req%target_i(1:req%max_size)=target_i(:)
    req%target_j(1:req%max_size)=target_j(:)
    
    req%max_size=req%max_size*2
         
    DEALLOCATE(src_domain)
    DEALLOCATE(src_ind)
    DEALLOCATE(target_ind)
    DEALLOCATE(src_i)
    DEALLOCATE(src_j)
    DEALLOCATE(target_i)
    DEALLOCATE(target_j)

  END SUBROUTINE reallocate_request

      
    SUBROUTINE request_add_point(ind,i,j,request,pos)
    USE domain_mod
    USE field_mod
    IMPLICIT NONE
      INTEGER,INTENT(IN)            ::  ind
      INTEGER,INTENT(IN)            :: i
      INTEGER,INTENT(IN)            :: j
      TYPE(t_request),POINTER :: request(:)
      INTEGER,INTENT(IN),OPTIONAL  :: pos
      
      INTEGER :: src_domain
      INTEGER :: src_iim,src_i,src_j,src_n,src_pos,src_delta
      TYPE(t_request),POINTER :: req
      TYPE(t_domain),POINTER :: d
      
      req=>request(ind)
      d=>domain(ind)
      
      IF (req%max_size==req%size) CALL reallocate_request(req)
      req%size=req%size+1
      IF (req%type_field==field_t) THEN
        src_domain=domain(ind)%assign_domain(i,j)
        src_iim=domain(src_domain)%iim
        src_i=domain(ind)%assign_i(i,j)
        src_j=domain(ind)%assign_j(i,j)

        req%target_ind(req%size)=(j-1)*d%iim+i
        req%src_domain(req%size)=src_domain
        req%src_ind(req%size)=(src_j-1)*src_iim+src_i
      ELSE IF (req%type_field==field_u) THEN
        IF (.NOT. PRESENT(pos)) STOP 'argument request_add_point non conforme'
        
        src_domain=domain(ind)%assign_domain(i,j)
        src_iim=domain(src_domain)%iim
        src_i=domain(ind)%assign_i(i,j)
        src_j=domain(ind)%assign_j(i,j)
        src_n=(src_j-1)*src_iim+src_i
        src_delta=domain(ind)%delta(i,j)
        
        src_pos=MOD(pos-1+src_delta+6,6)+1
        
        req%target_ind(req%size)=(j-1)*d%iim+i+d%u_pos(pos)
        req%src_domain(req%size)=src_domain
        req%src_ind(req%size)=src_n+domain(src_domain)%u_pos(src_pos)

        req%target_i(req%size)=i
        req%target_j(req%size)=j
        req%src_i(req%size)=domain(ind)%assign_i(i,j)
        req%src_j(req%size)=domain(ind)%assign_j(i,j)
        
!        PRINT *,ind,i,j,"src_delta",src_delta

      ELSE IF (req%type_field==field_z) THEN
        IF (.NOT. PRESENT(pos)) STOP 'argument request_add_point non conforme'

        src_domain=domain(ind)%assign_domain(i,j)
        src_iim=domain(src_domain)%iim
        src_i=domain(ind)%assign_i(i,j)
        src_j=domain(ind)%assign_j(i,j)
        src_n=(src_j-1)*src_iim+src_i
        src_delta=domain(ind)%delta(i,j)
        
        src_pos=MOD(pos-1+src_delta+6,6)+1
        
        req%target_ind(req%size)=(j-1)*d%iim+i+d%z_pos(pos)
        req%src_domain(req%size)=src_domain
        req%src_ind(req%size)=src_n+domain(src_domain)%z_pos(src_pos)
      ENDIF
  END SUBROUTINE request_add_point
  
 
  SUBROUTINE transfert_request(field,request)
  USE field_mod
  USE domain_mod
  IMPLICIT NONE
    TYPE(t_field),POINTER :: field(:)
    TYPE(t_request),POINTER :: request(:)
    REAL(rstd),POINTER :: rval2d(:) 
    REAL(rstd),POINTER :: rval3d(:,:) 
    REAL(rstd),POINTER :: rval4d(:,:,:) 
    INTEGER :: ind
    TYPE(t_request),POINTER :: req
    INTEGER :: n
    REAL(rstd) :: var1,var2
    
    DO ind=1,ndomain
      req=>request(ind)
      rval2d=>field(ind)%rval2d
      rval3d=>field(ind)%rval3d
      rval4d=>field(ind)%rval4d
      
      IF (field(ind)%data_type==type_real) THEN
        IF (field(ind)%ndim==2) THEN
          DO n=1,req%size
            rval2d(req%target_ind(n))=field(req%src_domain(n))%rval2d(req%src_ind(n))
          ENDDO
        ELSE IF (field(ind)%ndim==3) THEN
          DO n=1,req%size
            rval3d(req%target_ind(n),:)=field(req%src_domain(n))%rval3d(req%src_ind(n),:)
          ENDDO
        ELSE IF (field(ind)%ndim==4) THEN
          DO n=1,req%size
            rval4d(req%target_ind(n),:,:)=field(req%src_domain(n))%rval4d(req%src_ind(n),:,:)
          ENDDO
        ENDIF
      ENDIF        

    ENDDO
    
  END SUBROUTINE transfert_request

END MODULE transfert_mod
      
        
        
        
      
